/******************************************************************************
*                                                                             *
*                          Network Job Control (NJC)                          *
*                                                                             *
*                             Q U E U E C . C                                 *
*                                 queuec                                      *
*                               Version 1.0                                   *
*                                                                             *
*                   Copyright (c) Joerg-R. Hill, April 1992                   *
*                                                                             *
*-----------------------------------------------------------------------------*
* This is the client for Network Job Control. It has to run on all hosts in   *
* the network which are managed by NJC.                                       *
* Possible command line option: -debug                                        *
******************************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<unistd.h>
#ifdef AIX
#include<sys/select.h>
#endif
#include<sys/signal.h>
#include<sys/time.h>
#include<sys/types.h>
#include<rpc/rpc.h>

#include "queue.h"

/* Prototypes */
int main(int, char **);
int find_server(void);

/* Prototypes from startjob.c */
#ifdef IRIX
extern void *startjob(void *);
#else
extern void startjob(JOB *);
#endif

/* Prototypes from checkpids.c */
/* extern int *checkpids(int *); */
extern void *checkpids(void *);

/* Prototypes from error_msg.c */
extern void error_msg(int, char *);

/* Prototypes from util.c */
FILE *open_file(char *, char *, char *);

/* Prototypes from xdr.c */
extern bool_t xdr_intarr(XDR *, int *);
extern bool_t xdr_job(XDR *, JOB *);

char *prgname;
char queue_server[MAXLENHOSTNAME]="";
char hostname[MAXLENHOSTNAME];
int debug=FALSE, print_date=TRUE;

void *queueok(void) /* This function is called via RPC from */
{                   /* the clients to find out the server   */
  static int pid;

  pid=getpid();
  return(&pid);
}

int main(int argc, char *argv[])
{
  int chpid, onServer=FALSE;
  register int i;

  prgname=strrchr(argv[0],'/');         /* Find programme name */
  if (prgname == NULL) prgname=argv[0];
  else prgname++;

  if (getuid())         /* Check whether root has started the programme */
  {
    fprintf(stderr,"%s: ERROR: Only root can execute this programme.\n",
			 prgname);
    exit(-1);
  }

  for (i=1; i<argc; i++)                        /* Look for options */
    if (!strcmp(argv[i],"-debug")) debug=TRUE;  /* debug option     */

  if ((chpid=fork()) < 0)
  {
    error_msg(2, "");
    exit(-1);
  }
  else if (chpid == 0)
  {
    onServer=find_server();
    if ((registerrpc(NJCCLIENT, VERS, STARTJOB, (void *)startjob, xdr_job, xdr_void)) == -1)
    {
      fprintf(stderr, "%s: ERROR: Could not register service.\n", prgname);
	exit(-1);
    }
    if ((registerrpc(NJCCLIENT, VERS, CHECKPIDS, (void *)checkpids, xdr_intarr, xdr_intarr)) == -1)
    {
      fprintf(stderr, "%s: ERROR: Could not register service.\n", prgname);
	exit(-1);
    }
    if ((registerrpc(NJCCLIENT, VERS, QUEUECLIENT, (void *)queueok, xdr_void, xdr_int)) == -1)
    {
      fprintf(stderr, "%s: ERROR: Could not register service.\n", prgname);
      exit(-1);
    }
    svc_run();
  }
  return(0);     /* We should never arrive here */
}

int find_server(void)
{
  FILE *cfg;
  char hosts[MAXHOSTS][MAXLENHOSTNAME];
  char line[MAXLENLINE], *word;
  int n, res;
  register i=0, j=0;

  gethostname(hostname, MAXLENHOSTNAME);

  if ((cfg=open_file(NJCPATH, "queue.cfg", "r")) == NULL) exit(-1);
  while ((fgets(line, MAXLENLINE, cfg)) != NULL)
  {
    j++;
    if (*line == '#') continue;
    if ((word=strtok(line, " \t")) == NULL)
    {
	fprintf(stderr, "%s: ERROR in configuration file, line %d.\n", prgname, j);
	exit(-1);
    }
    strcpy(hosts[i++], word);
  }
  fclose(cfg);
  n=i-1;

  strcpy(queue_server, hostname);
  while (*queue_server == '\0')
  {
    for (i=0; i<=n; i++)
    {
      if (callrpc(hosts[i], NJCSERVER, VERS, QUEUESERVER, (xdrproc_t)xdr_void, 0,
	                    (xdrproc_t)xdr_int, (char *)(&res)) == RPC_SUCCESS)
	{
	  strcpy(queue_server, hosts[i]);
	  break;
      }
    }
    if (*queue_server == '\0')
	fprintf(stderr, "%s: WARNING: Cannot contact server. Retrying ...\n", prgname);
  }

  if (debug) fprintf(stderr, "%s: Server is %s.\n", prgname, queue_server);
  if (strcmp(hostname, queue_server))
    return(FALSE);
  else
    return(TRUE);
}
