/******************************************************************************
*                                                                             *
*                          Network Job Control (NJC)                          *
*                                                                             *
*                            R E A D _ P I D S . C                            *
*                    common routines for queued and submit                    *
*                                                                             *
*                   Copyright (c) Joerg-R. Hill, April 1992                   *
*                                                                             *
******************************************************************************/

#include<errno.h>
#include<pwd.h>
#include<signal.h>
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<time.h>
#include<unistd.h>
/*#ifdef AIX
#include<sys/select.h>
#endif */
#include<sys/stat.h>
#include<sys/types.h>
#include<sys/wait.h>
#include<rpc/rpc.h>

#include "queue.h"

/* Prototypes */
void read_config(void);
void read_pids(void);
void update_pids(int, int);
void cfg_error(int);

/* Prototypes from util.c */
FILE *open_file(char *, char *, char *);
int  move_file(char *, char *, char *);
int  setmode(char *, char *, int);
int  update_file(FILE *, char *);

/* Prototypes from checkpids.c */
extern int  *checkpids(int *);

/* Prototypes from error_msg.c */
extern void error_msg(int, char *);

/* Prototypes from submit.c */
extern void ende(int);

/* Prototypes from xdr.c */
extern bool_t xdr_intarr(XDR *, int *);

extern struct {
  char name[MAXLENHOSTNAME];
  int  maxjobs;
  int  maxallowed;
  int  nice;
  int  load;
  int  numpids;
  int  pids[MAXPROCESS];
} host[];
extern int numhosts;
extern char *prgname, hostname[];

void read_config(void)
{
  FILE *file;
  char *word;
  register int i=0, j=0;
  char line[MAXLENLINE];

  if ((file=open_file(NJCPATH, "queue.cfg", "r")) == NULL) return;
                                                 /* read configuration */

  while ((fgets(line, MAXLENLINE, file)) != NULL)
  {
    j++;
    if (*line == '#') continue;     /* lines starting with "#" are comments */
    if ((word=strtok(line, " \t")) == NULL) cfg_error(j);
    strcpy(host[i].name, word);          /* fill configuration in structure */
                                         /* "host"                          */
    if ((word=strtok(NULL, " \t")) == NULL) cfg_error(j);
    host[i].maxjobs=atoi(word);
    if ((word=strtok(NULL, " \t")) == NULL) cfg_error(j);
    host[i].maxallowed=atoi(word);
    if ((word=strtok(NULL, " \t")) == NULL) cfg_error(j);
    host[i].nice=atoi(word);
    host[i++].load=0;

    if (i > MAXHOSTS)                   /* check for overflow */
    {
      error_msg(1, "");
      ende(-1);
    }
  }
  numhosts=i-1;

  fclose(file);
}

void read_pids(void)
{                              /* this function reads the queue.running   */
  FILE *running;               /* file into memory and checks whether all */
  register int i, j, k, pid;   /* processes are still running             */
  int istat, pids[MAXPROCESS], *wrongpids=pids;
  int found;
  char compi[MAXLENHOSTNAME], line[MAXLENLINE], *word;

  for (i=0; i<=numhosts; i++)  /* clear "pids" in structure "host" */
  {
    for (j=0; j<MAXPROCESS; j++)
      host[i].pids[j]=0;
    host[i].numpids=0;
  }

  if ((running=open_file(NJCPATH, "queue.running", "r")) == NULL) return;
                                            /* open file */

  while( (fgets(line, MAXLENLINE, running)) != NULL)      /* read it   */
  { 
    word=strtok(line, ";");
    word=strtok(NULL, ";");
    strcpy(compi, word);                       /* get hostname and PID */
    word=strtok(NULL, ";");
    pid=atoi(word);

    found=FALSE;                       /* searching host in structure */
    for (i=0; i<=numhosts; i++)        /* "host"                      */
      if (!strcmp(host[i].name, compi))
      {
        found=TRUE;                                    /* we found it */
        break;
      }
    if (!found) continue;        /* we didn't find it, take next line */

    if (host[i].numpids > MAXPROCESS)           /* check for overflow */
    {
      error_msg(3, compi);
      ende(-1);
    }
    host[i].pids[host[i].numpids]=pid;    /* copy PID to structure "host" */
    host[i].numpids++;
  }
  fclose(running);

  for (i=0; i<=numhosts; i++)               /* go through list of hosts */
  {
    if (host[i].load == -1) continue;      /* host did not respond in a */
                                           /* former call via RPC       */
    for (j=0; j<=host[i].numpids-1; j++)  /* copy PID's to array "pids" */
      pids[j]=host[i].pids[j];
    if (j<=MAXPROCESS) pids[j]=0;

    if (strcmp(hostname, host[i].name))      /* if host is a remote one */
    {                                   /* send it array "pids" via RPC */
      if ((istat=callrpc(host[i].name, NJCCLIENT, VERS, CHECKPIDS, (xdrproc_t)xdr_intarr,
                         (char *)pids, (xdrproc_t)xdr_intarr, (char *)wrongpids)) != 0)
      {
        error_msg(5, host[i].name);
        clnt_perrno(istat);
        fprintf(stderr, "\n");
        wrongpids[0]=0;
      }
    }
    else                                    /* it is the local host */
      wrongpids=checkpids(pids);            /* call check function  */

    k=0;                          /* look at the results of the check */
    while (wrongpids[k] != 0 && k<=MAXPROCESS)
    {
      j=0;                                    /* there is a wrong PID */
      while (host[i].pids[j++] != wrongpids[k]);
      update_pids(i, --j);          /* remove it from "queue.running" */ 
      host[i].numpids--;
      k++;
    }
    if (host[i].maxjobs == 0)   /* hosts which do not accept requests */
      host[i].load=100;      /* should not be used, mark them as full */
    else
      host[i].load=100*host[i].numpids/host[i].maxjobs;  /* calculate */
  }   /* end of for (i= ... ) */                         /* load      */
}

void update_pids(int hostnum, int i)         /* this function updates the */
{                                            /* queue.running file        */
  FILE *running, *newrunning;
  char line[MAXLENLINE], saveline[MAXLENLINE], tmp[L_tmpnam], *word;
  register int j;
                                             /* Send message that something */
                                             /* was going wrong             */
  sprintf(line, "Process %d is not running on host %s.",
                 host[hostnum].pids[i], host[hostnum].name);
  error_msg(7, line);

  if ((running=open_file(NJCPATH, "queue.running", "r+")) == NULL) return;
  if ((newrunning=open_file("", tmp, "w")) == NULL) return;

  clearerr(newrunning);
  while ((fgets(line, MAXLENLINE, running)) != NULL)  /* and update file */
  {
    strcpy(saveline, line);
    word=strtok(line, ";");
    word=strtok(NULL, ";");
    if (strcmp(word, host[hostnum].name))
      fprintf(newrunning, "%s", saveline);
    else
    {
      j=atoi(strtok(NULL,";"));
      if (j != host[hostnum].pids[i])
        fprintf(newrunning, "%s", saveline);
    }
  } 
  j=ferror(newrunning);
  fclose(newrunning);
  if (j)
  {
    error_msg(12, tmp);
    return;
  }
  if (!update_file(running, tmp))
  {
    error_msg(12, "queue.running");
    return;
  }
}

void cfg_error(int i)          /* This function is called when an error */
{                              /* was found in "queue.cfg"              */
  fprintf(stderr, "%s: ERROR in configuration file, line %d.\n", prgname, i);
  ende(-1);
}
