/******************************************************************************
*                                                                             *
*                          Network Job Control (NJC)                          *
*                                                                             *
*                               U T I L . C                                   *
*                common routines for queuec, queued, and submit               *
*                                                                             *
*                   Copyright (c) Joerg-R. Hill, April 1992                   *
*                                                                             *
******************************************************************************/

#include<stdio.h>
#include<string.h>
#include<unistd.h>
#ifdef AIX
#include<sys/access.h>
#endif
#include<sys/stat.h>
#include<time.h>

#include "queue.h"

/* Prototypes */
FILE *open_file(char *, char *, char *);
FILE *reopen_file(char *, char *, char *, FILE *);
int  setmode(char *, char *, int);
int  move_file(char *, char *, char *);
int  rm(char *, char *);
int  exist(char *, char *);
int  filelength(char *, char *);
int  update_file(FILE *, char *);
char *gettime(void);

extern char *prgname;

FILE *open_file(char *path, char *name, char *modus)
{
  FILE *fileptr;
  char line[MAXLENLINE];

  if (*path == (char)NULL)
  {
    tmpnam(line);
    strcpy(name, line);
  }
  else
  {
    strcpy(line, path);
    strcat(line, "/");
    strcat(line, name);
  }

  if ((fileptr=fopen(line, modus)) == NULL)
  {
    fprintf(stderr, "%s: %s: ERROR: Unable to open file %s. ", gettime(),
            prgname, line);
    perror((char *)NULL);
  }
  return(fileptr);
}

FILE *reopen_file(char *path, char *name, char *modus, FILE *file)
{
  FILE *fileptr;
  char line[MAXLENLINE];

  strcpy(line, path);
  strcat(line, "/");
  strcat(line, name);

  if ((fileptr=freopen(line, modus, file)) == NULL)
  {
    fprintf(stderr, "%s: %s: ERROR: Unable to reopen file %s. ", gettime(),
            prgname, line);
    perror((char *)NULL);
  }
  return(fileptr);
}

int setmode(char *path, char *name, int mode)
{
  char line[MAXLENLINE];

  strcpy(line, path);
  strcat(line, "/");
  strcat(line, name);

  if (chmod(line, mode))
  {
    fprintf(stderr, "%s: %s: ERROR: Cannot chmod %s. ", gettime(),
            prgname, line);
    perror((char *)NULL);
    return(FALSE);
  }
  return(TRUE);
}

int move_file(char *path, char *name1, char *name2)
{
  char file1[MAXLENLINE], file2[MAXLENLINE];

  strcpy(file1, path);
  strcat(file1, "/");
  strcat(file1, name1);

  strcpy(file2, path);
  strcat(file2, "/");
  strcat(file2, name2);

  if (rename(file1, file2))
  {
    fprintf(stderr, "%s: %s: ERROR: Unable to rename file %s. ", gettime(),
            prgname, file1);
    perror((char *)NULL);
    return(FALSE);
  }
  return(TRUE);
}

int rm(char *path, char *name)
{
  char line[MAXLENLINE];

  strcpy(line, path);
  strcat(line, "/");
  strcat(line, name);

  if (remove(line))
  {
    fprintf(stderr, "%s: %s: ERROR: Unable to remove file %s. ", gettime(),
            prgname, line);
    perror((char *)NULL);
    return(FALSE);
  }
  return(TRUE);
}

int exist(char *path, char *name)
{
  char line[MAXLENLINE];

  strcpy(line, path);
  strcat(line, "/");
  strcat(line, name);

  return(!access(line, 0));
}

int filelength(char *path, char *name)
{
  struct stat filestat;
  char line[MAXLENLINE];

  strcpy(line, path);
  strcat(line, "/");
  strcat(line, name);

  if (stat(line, &filestat))
  {
    fprintf(stderr, "%s: %s: ERROR: Unable to determine length of file %s. ", gettime(),
            prgname, line);
    perror((char *)NULL);
    return(-1);
  }

  return(filestat.st_size);
}

int update_file(FILE *which, char *scratch)
{
  FILE *help;
  char line[MAXLENLINE];
  int i;

  rewind(which);
  ftruncate(fileno(which), 0);

  clearerr(which);
  if ((help=fopen(scratch, "r")) == NULL)
  {
    fprintf(stderr, "%s: %s: ERROR: Unable to open file %s. ", gettime(),
            prgname, scratch);
    perror((char *)NULL);
    return(FALSE);
  }

  while ((fgets(line, MAXLENLINE, help)) != NULL) fprintf(which, "%s", line);
  i=ferror(which);
  fclose(which);
  fclose(help);

  if (i) return(FALSE);

  if (remove(scratch))
  {
    fprintf(stderr, "%s: %s: ERROR: Unable to remove file %s. ", gettime(),
            prgname, scratch);
    perror((char *)NULL);
    return(FALSE);
  }
  return(TRUE);
}

char *gettime(void)
{
  time_t now;
  static char t[26];

  now=time((time_t *)NULL);
  strcpy(t, ctime(&now));
  t[24]='\0';
  return(t);
}
